jQuery.noConflict();

var Screenpush = (function() {

      // Constants
      var appName = "Screenpush";
      var winEvents = ["scroll","resize"];
      var domEvents = ["DOMAttrModified", "DOMNodeInserted", "DOMNodeRemoved","change","focus","keypress","click"];
      var skinRoot = "chrome://screenpush/skin/";

      var statuses = {
        normal: { icon: "screenpush_status_icon.gif", tooltip: "Not Connected"},
        connected: { icon: "screenpush_status_icon_connected.gif", tooltip: "Connected to Cometd"},
        transmitting: { icon: "screenpush_status_icon_transmit.gif", tooltip: "Contacting Server"},
        error: { icon: "screenpush_status_icon_error.gif", tooltip: "Connection Error"},
      }
    
      // Private vars
      var canvas = null;
      var panel = null;
      var pointer = null;

      var prefs = null;
      var connected = false;
      var timer = null;
      var lastImgUrl = null;

      var autopush = false;
      var trackPointer = false;
      var mouseCoords = {x:-1,y:-1};

      // Local copies of prefs
      var cometdUrl = null;
      var cometdChannel = null;
      var useJpeg = null;
      var jpegQuality = null;
      var pushDelay = null;

      var connect = function() {
        jQuery.comet.init(cometdUrl);
        jQuery.comet.subscribe("/cometd/meta",onMeta);
      };


      // Main screengrab + push functionality
      var pushScreen = function() {

        if (timer) {
          clearTimeout(timer);
          timer = null;
        }       

        if (!connected) {
          connect();
        }

        var win = gBrowser.selectedBrowser.contentWindow;

        // Get the DOM element with window dimensions and scroll info
        var clientInfo = win.document.compatMode == "CSS1Compat" ? 
                            win.document.documentElement :
                            win.document.body;

        if (!clientInfo) return;

        var region = {x: win.scrollX, y: win.scrollY, 
                      width: clientInfo.clientWidth,
                      height: clientInfo.clientHeight };

        // Size the canvas
        canvas.width = region.width;
        canvas.height = region.height;
        canvas.style.width = canvas.style.maxwidth = region.width + "px";
        canvas.style.height = canvas.style.maxheight = region.height + "px";
        
        // Render window onto canvas
        var context = canvas.getContext("2d");
        context.clearRect(region.x, region.y, region.width, region.height);
        context.drawWindow(win, region.x, region.y, region.width, region.height, "#000");

        // Composite on a fake pointer if needed
        if (autopush && trackPointer && mouseCoords.x>=0 && mouseCoords.y >= 0 && 
            mouseCoords.x < region.width && mouseCoords.y < region.height) {

          context.save();
          context.shadowColor = "rgba(0,0,0,0.6)";
          context.shadowOffsetY = 1;
          context.shadowBlur = 2;
          context.drawImage(pointer, mouseCoords.x, mouseCoords.y);
          context.restore();
        }

        // Get data URL of rendered canvas
        var dataUrl = useJpeg ? canvas.toDataURL("image/jpeg","quality="+jpegQuality) 
                                   : canvas.toDataURL("image/png");

        // Only transmit it if image is different to last time
        if (dataUrl != lastImgUrl) {
          lastImgUrl = dataUrl;
          jQuery.comet.publish(cometdChannel, { dataUrl: dataUrl });

          setPanelStatus("transmitting");
        }
      };

      var setPanelStatus = function(status,msg) {
        panel.src=skinRoot+statuses[status].icon;
        panel.tooltipText=appName+" - "+statuses[status].tooltip + (msg?': '+msg:'');
      }

      var getWindows = function() {
        var windows = [];
        for (var i = 0; i < gBrowser.browsers.length; i++) {
          windows[i]=gBrowser.getBrowserAtIndex(i).contentWindow;
        }

        return windows;
      }

      var startListeningToDOMEvents = function() {
        getWindows().forEach(function(win) {
          for (var i=0;i<winEvents.length;i++) {
            win.addEventListener(winEvents[i], onPushEvent, false);
          }
          for (var i=0;i<domEvents.length;i++) {
            win.document.addEventListener(domEvents[i], onPushEvent, true);
          }
        });

        gBrowser.addEventListener("load", onPushEvent, true);
        gBrowser.mPanelContainer.addEventListener("select", onPushEvent, false);
      };

      var stopListeningToDOMEvents = function() {
        getWindows().forEach(function(win) {
          for (var i=0;i<winEvents.length;i++) {
            win.removeEventListener(winEvents[i], onPushEvent, false);
          }
          for (var i=0;i<domEvents.length;i++) {
            win.document.removeEventListener(domEvents[i], onPushEvent, true);
          }
        });

        gBrowser.mPanelContainer.removeEventListener("select", onPushEvent, false);
      };

      
      var startListeningToMouseEvents = function () {
        getWindows().forEach(function(win) {
            win.document.addEventListener("mousemove", onMouseMoved, true);
        });
      };

      var stopListeningToMouseEvents = function () {
        getWindows().forEach(function(win) {
            win.document.removeEventListener("mousemove", onMouseMoved, true);
        });
      };

      var dropCometConnection = function() {
        if (connected) {
          jQuery.comet.disconnect();
          jQuery.comet.unsubscribe("/cometd/meta");
          connected=false;
        }
        setPanelStatus("normal");
        lastImgUrl = null;
        if (autopush) {
          pushScreen();
        }
      };


      // Private event handlers
      var onPageLoad = function(event) {
        if (autopush) {
          if (event.originalTarget instanceof HTMLDocument) {

            // TODO: only need to do these for current page
            // ... but adding is idempotent, so leave it for now
            startListeningToDOMEvents();            
            if (trackPointer) {
              startListeningToMouseEvents();
            }

            if (gBrowser.getBrowserForDocument(event.originalTarget) == gBrowser.selectedBrowser) {
              // Don't trigger on loading in another tab
              // BUG: this condition doesn't work - listening to wrong event?
              onPushEvent(event);
            }
          }
        }
      };

      var onMouseMoved = function(e) {
        mouseCoords = {x:e.clientX,y:e.clientY};
        if (autopush) {
          onPushEvent(e);
        }
      };

      var onMeta = function(e,msg) {

        var meta=msg.data;
        if ((meta.action == "connect" || meta.action == "handshake")
                && !meta.successful) {
          setPanelStatus("error",meta.errorCode);
          connected = false;      
        } else if (meta.action == "connect" && meta.successful) {
          setPanelStatus("connected");
          connected = true;
        } else if (meta.action == "publish") {
          meta.successful ? setPanelStatus("connected") : setPanelStatus("error",meta.errorCode);
        }
      };

      var onPushEvent = function(e) {

        // Do nothing if already a push pending
        if (timer) return;

        // Ignore event if it's a DOM event from another tab
        if(e.currentTarget instanceof HTMLDocument && 
            e.currentTarget != gBrowser.selectedBrowser.contentWindow.document) {
          return;
        }

        // Else push screen soon 
        timer = setTimeout(pushScreen, pushDelay);
      };


      // Public members - callbacks from Chrome must be exposed
      return {
        onExtensionLoad: function() {
          canvas = document.getElementById("screenpush_buffer_canvas");
          panel = document.getElementById("screenpush_panel");
          pointer = document.getElementById("screenpush_mouse_pointer");

          // Register to receive notifications of preference changes
          prefs = Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService)
            .getBranch("extensions.screenpush.");

          prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
          prefs.addObserver("", this, false);

          cometdUrl = prefs.getCharPref("cometdUrl");
          cometdChannel = prefs.getCharPref("cometdChannel");
          useJpeg = prefs.getIntPref("imageType") == 1;
          jpegQuality = prefs.getIntPref("jpegQuality");
          pushDelay = prefs.getIntPref("pushDelay");

          gBrowser.addEventListener("DOMContentLoaded", onPageLoad, true);
        },


        onExtensionUnload: function() {
          if (connected) {
            jQuery.comet.disconnect();
          }
          prefs.removeObserver("", this);
        },

        // Listen for prefs changes
        observe: function(subject, topic, data) {
          if (topic != "nsPref:changed") {
           return;
          }

          switch(data) {
            case "cometdUrl":
              cometdUrl = prefs.getCharPref("cometdUrl");
              dropCometConnection();
            break;

            case "cometdChannel":
              cometdChannel = prefs.getCharPref("cometdChannel");
            break;

            case "imageType":
              useJpeg = (prefs.getIntPref("imageType") == 1);
            break;

            case "jpegQuality":
              jpegQuality = prefs.getIntPref("jpegQuality");
            break;

            case "pushDelay":
              pushDelay = prefs.getIntPref("pushDelay");
            break;
          }
        },

        statusPanelClicked: function() {
          // Bypass cache
          lastImgUrl = null;
          pushScreen(true);
        },

        autoPushChecked: function(event) {
          if (event.target.hasAttribute("checked")) {
            autopush = true;
            startListeningToDOMEvents();

            document.getElementById("track-pointer-menuitem").setAttribute("disabled",false);
            if (trackPointer) {
              startListeningToMouseEvents();
            }

            pushScreen();
          } else {
            autopush = false;
            stopListeningToDOMEvents();

            document.getElementById("track-pointer-menuitem").setAttribute("disabled",true);
            if (trackPointer) {
              stopListeningToMouseEvents();
            }
          }
        },

        trackPointerChecked: function(event) {
          if (event.target.hasAttribute("checked")) {
            trackPointer = true;
            startListeningToMouseEvents();
          } else {
            trackPointer = false;
            stopListeningToMouseEvents();
          }
        },

        showOptionsDialog: function() {

          // From https://wiki.mozilla.org/XUL:Windows
          var instantApply = getBoolPref("browser.preferences.instantApply", false);
          var features = "chrome,titlebar,toolbar,centerscreen" + (instantApply ? ",dialog=no" : ",modal");
          var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
           .getService(Components.interfaces.nsIWindowMediator);
          var win = wm.getMostRecentWindow("Browser:Preferences");
          openDialog("chrome://screenpush/content/options.xul",
              "Screenpush Options", features, Screenpush);

        },

        restartConnection: function() {
          dropCometConnection();
        }
      }
})();

window.addEventListener("load", function(e) { Screenpush.onExtensionLoad(e); }, false); 
window.addEventListener("unload", function(e) { Screenpush.onExtensionUnload(e); }, false); 
